/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/
#ifndef __BLAST_DBASE__
#define __BLAST_DBASE__
#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
		BLAST_DB_FMT_NONE=0,
		BLAST_DB_FMT_FASTA=1,
		BLAST_DB_FMT_ASN1=2,
		BLAST_DB_FMT_BLAST=3,
		BLAST_DB_FMT_IG=4
	} BLAST_DBFormat, PNTR BLAST_DBFormatPtr;


typedef struct blast_dbprocset {
		BLAST_DBFilePtr	(LIBCALLBACK *open) PROTO((CharPtr name, BLAST_AlphabetPtr));
		void	(LIBCALLBACK *close) PROTO((BLAST_DBFilePtr));
		BLAST_Error	(LIBCALLBACK *getseq) PROTO((BLAST_DBFilePtr, BLAST_SeqPtr));
		BLAST_StrPtr	(LIBCALLBACK *curname) PROTO((BLAST_DBFilePtr));
		BLAST_DBFilePtr	(LIBCALLBACK *link) PROTO((BLAST_DBFilePtr));
		BLAST_Error (LIBCALLBACK *seek) PROTO((BLAST_DBFilePtr));
		BLAST_Error (LIBCALLBACK *rewind) PROTO((BLAST_DBFilePtr));

		struct blast_dbprocset PNTR next;
		BLAST_DBFormat	format;	/* FASTA, BLAST, ASN.1 */
		BLAST_AlphabetPtr	ap;	/* protein or nucleic acid */
		long	version_min, version_max;	/* versions of format */
	} BLAST_DBProcSet, PNTR BLAST_DBProcSetPtr;

typedef struct {
		int		error;
		long	recid;
		VoidPtr		procsetdata;
		struct blast_dbprocset	dbps;
		long	nrec;
		long	totlen;
		Boolean	seekable;
		long	version;
	} BLAST_DBFile, PNTR BLAST_DBFilePtr;


BLAST_DBProcSetPtr LIBCALL BlastDBProcSetFind PROTO((BLAST_Alphatype alphatype, BLAST_DBFormat format, long version));
BLAST_DBProcSetPtr LIBCALL BlastDBProcSetFindNext PROTO((BLAST_DBProcSetPtr, long, long, long));
BLAST_Error LIBCALL BlastDBProcSetRegister PROTO((BLAST_DBProcSetPtr));

BLAST_DBFilePtr LIBCALL BlastDBOpen PROTO((CharPtr name, BLAST_AlphabetPtr ap, long format, CharPtr mode));
void LIBCALL 	BlastDBClose PROTO((BLAST_DBFilePtr));
BLAST_Error LIBCALL BlastDBGetSeq PROTO((BLAST_DBFilePtr, BLAST_SeqPtr));
BLAST_StrPtr LIBCALL BlastDBCurname PROTO((BLAST_DBFilePtr));

#ifdef __cplusplus
}
#endif
#endif /* !__BLAST_DBASE__ */
