/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#ifndef __BLAST_ERROR__
#define __BLAST_ERROR__
#ifdef __cplusplus
extern "C" {
#endif

enum _blast_error {
		BLAST_ERR_NONE = 0,
		BLAST_ERR_UNDEF, /* undefined (general) error */
		BLAST_ERR_INVAL,	/* invalid parameter value provided */
		BLAST_ERR_DOMAIN,	/* domain error on parameter */
		BLAST_ERR_RANGE,	/* range error on parameters */
		BLAST_ERR_MEM,	/* insufficient memory */
		BLAST_ERR_EXTENT,	/* extent of request is too great */
		BLAST_ERR_POP,	/* no more states to pop */
		BLAST_ERR_FFORMAT,	/* file format error */
		BLAST_ERR_CONV,		/* data conversion error */
		BLAST_ERR_NF,		/* object not found */
		BLAST_ERR_INIT,		/* object not initialized */
		BLAST_ERR_ALPHATYPE,	/* incorrect type of alphabet */
		BLAST_ERR_ALPHASIZE,	/* bad alphabet size (too large or <= 0) */
		BLAST_ERR_ALPHAVAL,		/* disallowed letter value */
		BLAST_ERR_SENTINEL,		/* bad or missing sentinel */
		BLAST_ERR_ALPHACONFLICT,	/* conflicting alphabets */
		BLAST_ERR_LETTER,		/* Invalid letter value */
		BLAST_ERR_RESFREQ,		/* resfreq calculation error */
		BLAST_ERR_SCORE_RANGE,	/* range of scoring is invalid */
		BLAST_ERR_SCORE_DOMAIN,	/* domain of score is invalid */
		BLAST_ERR_SCORE_AVG,	/* average expected score is non-negative */
		BLAST_ERR_SCOREBLK_TYPE,	/* bad BLAST_ScoreBlkType */
		BLAST_ERR_DFA,			/* error in dfa library */
		BLAST_ERR_WORDSIZE,		/* invalid wordsize */
		BLAST_ERR_CONTEXT,		/* too many contexts or invalid context */
		BLAST_ERR_WFSTATE,		/* WordFinder is in an incompatible state */
		BLAST_ERR_WFTYPE,		/* WordFinder type mismatch */
		BLAST_ERR_ENCODING,		/* invalid alphabet encoding requested */
		BLAST_ERR_QUERYLEN,		/* unsupported query sequence length */
		BLAST_ERR_SUBJECTLEN,	/* unsupported subject sequence length */
		BLAST_ERR_APPLICATION,	/* application error */
		BLAST_ERR_MAX
	};

typedef enum _blast_error	BLAST_Error, PNTR BLAST_ErrorPtr;

BLAST_EXTERN BLAST_Error	blast_errno;

char *BlastErrStr PROTO((BLAST_Error));

#ifdef __cplusplus
}
#endif
#endif /* !__BLAST_ERROR__ */
